package core

import (
	"regexp"
	"strings"
)

// PathMapSeparatorStr specifies the separator string to recognise in path mappings
const requestRemapSeparatorStr = " -> "

// RequestRemap is a structure to hold a remap regex to check against, and a template to apply this transformation onto
type RequestRemap struct {
	Regex    *regexp.Regexp
	Template string
}

// compileCGIRegex takes a supplied string and returns compiled regular expression
func compileCGIRegex(cgiDir string) *regexp.Regexp {
	SystemLog.Infof("CGI directory: %s", cgiDir)
	return regexp.MustCompile("^" + cgiDir + "(/.*)?$")
}

// compileRestrictedPathsRegex turns a string of restricted paths into a slice of compiled regular expressions
func compileRestrictedPathsRegex(restrictions []string) []*regexp.Regexp {
	regexes := make([]*regexp.Regexp, 0)

	// Split restrictions string by new lines
	for _, expr := range restrictions {
		// Skip empty expressions
		if len(expr) == 0 {
			continue
		}

		// Compile the regular expression
		regex, err := regexp.Compile("^" + expr + "$")
		if err != nil {
			SystemLog.Fatalf("Failed compiling restricted path regex: %s", expr)
		}

		// Append compiled regex and log
		regexes = append(regexes, regex)
		SystemLog.Infof("Compiled restricted path regex: %s", expr)
	}

	return regexes
}

// compileRestrictedPathsRegex turns a string of hidden paths into a slice of compiled regular expressions
func compileHiddenPathsRegex(hidden []string) []*regexp.Regexp {
	regexes := make([]*regexp.Regexp, 0)

	// Split restrictions string by new lines
	for _, expr := range hidden {
		// Skip empty expressions
		if len(expr) == 0 {
			continue
		}

		// Compile the regular expression
		regex, err := regexp.Compile("^" + expr + "$")
		if err != nil {
			SystemLog.Fatalf("Failed compiling hidden path regex: %s", expr)
		}

		// Append compiled regex and log
		regexes = append(regexes, regex)
		SystemLog.Infof("Compiled hidden path regex: %s", expr)
	}

	return regexes
}

// compileRequestRemapRegex turns a string of remapped paths into a slice of compiled RequestRemap structures
func compileRequestRemapRegex(remaps []string) []*RequestRemap {
	requestRemaps := make([]*RequestRemap, 0)

	// Split remaps string by new lines
	for _, expr := range remaps {
		// Skip empty expressions
		if len(expr) == 0 {
			continue
		}

		// Split into alias and remap
		split := strings.Split(expr, requestRemapSeparatorStr)
		if len(split) != 2 {
			SystemLog.Fatalf("Invalid request remap regex: %s", expr)
		}

		// Compile the regular expression
		regex, err := regexp.Compile("^" + strings.TrimPrefix(split[0], "/") + "$")
		if err != nil {
			SystemLog.Fatalf("Failed compiling request remap regex: %s", expr)
		}

		// Append RequestRemap and log
		requestRemaps = append(requestRemaps, &RequestRemap{regex, strings.TrimPrefix(split[1], "/")})
		SystemLog.Infof("Compiled path remap regex: %s", expr)
	}

	return requestRemaps
}

// withinCGIDirEnabled returns whether a Path's relative value matches within the CGI dir
func withinCGIDirEnabled(p *Path) bool {
	return cgiDirRegex.MatchString(p.Relative())
}

// withinCGIDirDisabled always returns false, CGI is disabled
func withinCGIDirDisabled(p *Path) bool {
	return false
}

// isRestrictedPathEnabled returns whether a Path's relative value is restricted
func isRestrictedPathEnabled(p *Path) bool {
	for _, regex := range restrictedPaths {
		if regex.MatchString(p.Selector()) {
			return true
		}
	}
	return false
}

// isRestrictedPathDisabled always returns false, there are no restricted paths
func isRestrictedPathDisabled(p *Path) bool {
	return false
}

// isHiddenPathEnabled returns whether a Path's relative value should be hidden
func isHiddenPathEnabled(p *Path) bool {
	for _, regex := range hiddenPaths {
		if regex.MatchString(p.Selector()) {
			return true
		}
	}
	return false
}

// isHiddenPathDisabled always returns false, there are no hidden paths
func isHiddenPathDisabled(p *Path) bool {
	return false
}

// remapRequestEnabled tries to remap a request, returning bool as to success
func remapRequestEnabled(request *Request) bool {
	for _, remap := range requestRemaps {
		// No match, gotta keep looking
		if !remap.Regex.MatchString(request.Path().Selector()) {
			continue
		}

		// Create new request from template and submatches
		raw := make([]byte, 0)
		for _, submatches := range remap.Regex.FindAllStringSubmatchIndex(request.Path().Selector(), -1) {
			raw = remap.Regex.ExpandString(raw, remap.Template, request.Path().Selector(), submatches)
		}

		// Split to new path and query again
		path, query := SplitBy(string(raw), "?")

		// Remap request, log, return
		request.Remap(path, query)
		return true
	}
	return false
}

// remapRequestDisabled always returns false, there are no remapped requests
func remapRequestDisabled(request *Request) bool {
	return false
}
